/*
** BasicMigration.java
**
** Copyright (c) 2017 CA.  All rights reserved.
** This software may not be duplicated, disclosed or reproduced in whole or
** in part for any purpose except as authorized by the applicable license agreement,
** without the express written authorization of CA. All authorized reproductions
** must be marked with this language.
**
** TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS
** SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING
** WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT
** WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS
** OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL,
** INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS
** INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY
** ADVISED OF SUCH LOSS OR DAMAGE.
*/

/*
 *       CA Single Sign-On REST API Migration Sample.
 *
 *        Policy Migration Client App (Java)
 *
 * This sample program demonstrates how to create, read, update, and delete
 * policy objects in policy environments using simple calls to the Policy Data REST API.
 * It also demonstrates granular migration (export and import) using calls to the Policy Migration REST API.
 * All REST calls need a security token.  This token is fetched from the
 * Authentication Service, which accepts the administrator's basic credentials - username/password.
 * The Authentication Service authenticates the administrator with the Policy Server and
 * returns the corresponding session spec.  This is embedded inside a JWT and returned
 * to the client.
 *
 * Usage: see README and included 'build' and 'run' scripts
 *
 */
import java.util.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.lang.NullPointerException;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.util.Properties;
import java.lang.StringBuilder;
import java.net.URL;

import sample.restclient.Client;
import sample.restclient.SmObject;
import sample.restclient.ObjectData;
import sample.restclient.LinkList;
import sample.restclient.LinkMap;
import sample.restclient.ObjectLink;
import sample.restservice.BasicService;
import sample.restservice.RequestType;
import sample.restclient.SessionInfo;

/**
 * This class contains the main engine of the migration sample app.
 */
public class BasicMigration {

    // Strings to indicate which command to use
    public static final String EXPORT = "export";
    public static final String IMPORT = "import";
    public static final String MANAGE = "manage";

    // Command Line Switches
    public static final String AUTHENDPOINT_SWITCH = "-ap";
    public static final String RESTENDPOINT_SWITCH = "-ep";
    public static final String PASSPHRASE_SWITCH = "-pphrase";
    public static final String FILE_SWITCH = "-file";
    
    // Index of each argument passed into main args array
    public static final int IOF_COMMAND = 0;
    public static final int IOF_AP = 1;
    public static final int IOF_AUTH = 2;
    public static final int IOF_EP = 3;
    public static final int IOF_REST = 4;
    public static final int IOF_PPHRASE = 5;
    public static final int IOF_PASSPHRASE = 6;
    public static final int IOF_FILE = 7;
    public static final int IOF_FILENAME = 8;

    // Logging
    public static final String INVALID_INPUT = "\nInvalid input. Try again.\n";
    public static final String AUTHNSVC_LOGIN_FAILED = "Authentication Service Login Failed. Make sure to enter the correct authentication endpoint, username, and password.\n";
    public static final String RETRY_OPTIONS = "Or, enter credentials manually by running in interactive mode.\n";
    public static final String HTTPS_REQUIRED = "\nHTTP not supported by the Rest Service. HTTPS Required.\n";
    
    // constants for logging
    private static final int SIZE = 4;
    private static final int REQ_HDR = 0;
    private static final int RES_HDR = 2;
    private static final int RES_BODY = 3;

    // other constants
    public static final String BEARER = "Bearer ";
    public static final String HTTP_PREFIX = "http://";
    public static final String HTTPS_PREFIX = "https://";

    // Session Specification of the administrator
    private String sessionSpec;

    // a BufferedWriter for logging
    private static BufferedWriter logging;

    // used for reading user-entered or automated input
    private BufferedReader inputReader;

    // This class contains the strings that are parsed from the command line input
    private class ParseArgs {

        public String command;
        public String authnSvcHost;
        public String authnSvcBaseUrl;
        public String hostname;
        public String baseUrl;
        public String passPhrase;
        public String fileName;

        public ParseArgs() {
            command = null;
            authnSvcHost = null;
            authnSvcBaseUrl = null;
            hostname = null;
            baseUrl = null;
            passPhrase = null;
            fileName = null;
        }
        public void trim() {
            if(null != command) command.trim();
            if(null != authnSvcHost) authnSvcHost.trim();
            if(null != authnSvcBaseUrl) authnSvcBaseUrl.trim();
            if(null != hostname) hostname.trim();
            if(null != baseUrl) baseUrl.trim();
            if(null != passPhrase) passPhrase.trim();
            if(null != fileName) fileName.trim();
        }
    }

    public BasicMigration() {
        print("---------------------------------------------------------------------------");
        print("\n[CA Single Sign-On REST API - Policy Migration sample]");

        inputReader = new BufferedReader(new InputStreamReader(System.in));

        File file;
        logging = null;

        try {
            file = new File("output.txt");
            if (!file.exists())
                file.createNewFile();

            print("\nOutput file created at " + file.getAbsolutePath());
            logging = new BufferedWriter(new FileWriter(file, false));

        } catch (IOException e) {
            print("Failed to initialize log file: " + e);
        }
    }

    // prompt user for password and hide from console
    private char[] readPassword(String prompt) throws IOException {
        if (System.console() != null) {
            return System.console().readPassword(prompt);
        } else {
            return readLine(prompt).toCharArray();
        }
    }

    // read a line from system input
    private String readLine(String prompt) throws IOException {
        if (System.console() != null) {
            return System.console().readLine(prompt);
        } else {
            print(prompt);
            BufferedReader reader = inputReader;
            String line = reader.readLine();
            return line;
        }
    }

    private void readFile(File infile, StringBuilder builder) throws IOException {
        FileInputStream fis = new FileInputStream(infile);
        BufferedReader br = new BufferedReader(new InputStreamReader(fis));
        String inputline = null;
        while ((inputline = br.readLine()) != null) {
            builder.append(inputline);
            builder.append("\n");
        }

        br.close();
    }

    /*
     * Prompts the administrator for credentials - username, pwd - and sends a
     * request to the to the Authentication Service to fetch a JWT embedding the
     * administrator's session spec.'
     *
     * @param authnSvcHostName Host information about the Authentication Service.
     *
     * @param authnSvcBaseURL The base URL of the Authentication Service.
     *
     * @return Authentication attempt status
     */
    public boolean performAuthentication(String authnSvcHostName, String authnSvcBaseURL, boolean useProperties) {
        LogFile("===========================================================================");
        LogFile("    Make request for session token. Credentials hidden for security");
        LogFile("===========================================================================");

        String user = null;
        char[] psswd = null;
        //use either command line prompts or properties file to read in credentials
        if (!useProperties) {
            // prompt user for username
            try {
                user = readLine("Username:\n");
            } catch (IOException e) {
                print("Failed to read username: " + e);
                return false;
            }
            if (user == null || user.equals("")) {
                print(INVALID_INPUT);
                return false;
            }

            // prompt the user for password and hide it from the terminal window
            try {
                psswd = readPassword("Password:\n");
            } catch (IOException e) {
                print("Failed to read password: " + e);
                return false;
            }

            if (psswd == null || psswd.equals("")) {
                print(INVALID_INPUT);
                return false;
            }

        } else {
            //get admin credentials from properties file
            Properties prop = new Properties();
            InputStream input = null;
            try {
                input = new FileInputStream("BasicMigration.properties");
                prop.load(input);
                user = prop.getProperty("ADMIN_NAME");
                psswd = prop.getProperty("ADMIN_PWD").toCharArray();
                if (user == null || psswd == null) {
                    print("Error parsing properties. Enter correct property names in properties file.");
                    print(RETRY_OPTIONS);
                    usageMessage();
                    return false;
                }

            } catch (IOException e){
                print("Error reading properties file:\n" + e + "\n\nVerify correct properties file exists.");
                print(RETRY_OPTIONS);
                usageMessage();
                return false;
            } catch (NullPointerException n) {
                print("Error reading properties file. Enter correct property names in properties file.");
                print(RETRY_OPTIONS);
                usageMessage();
                return false;
            }
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    print("Error closing properties file: " + e);
                }
            }
        }
        print("Authenticating...\n");
        // fetch the JWT from the Authentication Svc
        sessionSpec = getJwt(authnSvcHostName, authnSvcBaseURL, user, new String(psswd));
        // zero out password once it's not needed
        Arrays.fill(psswd, ' ');

        if (null == sessionSpec) {
            print(AUTHNSVC_LOGIN_FAILED);
            return false;
        }

        print("Logged into REST Server at " + authnSvcHostName);
        return true;
    } // end performAuthentication

    /*
     * Fetches the JWT from the Authentication Service containing the sesson
     * spec corresponding to the Admin's credentials. Returns the session spec
     * of the Admin.
     *
     * @param authnSvcHostName Host information about the Authn Service.
     *
     * @param authnSvcBaseURL The base URL of the Authn Service.
     *
     * @param adminUser Admin's user name.
     *
     * @param passwd Admin's password.
     *
     * @return The session spec if successful otherwise null
     */
    private String getJwt(String authnSvcHostName, String authnSvcBaseURL, String adminUser, String passwd) {

        String sessionSpec;
        String hostinfo = authnSvcHostName;
        String basicAuthnB64;
        String response;

        // create the Base64(adminUser:passwd) - this will be sent to the
        // Authentication Service in the
        // Authorization header of the GET request
        String basicCreds = adminUser + ":" + passwd;

        try {
            basicAuthnB64 = Base64.getEncoder().encodeToString(basicCreds.getBytes("utf-8"));
        } catch (Exception e) {
            print("getJwt() - Base64.getEncoder exception " + e);
            return null;
        }

        if ((null == basicAuthnB64) || (basicAuthnB64.isEmpty())) {
            print("getJwt() - Base64.getEncoder retuned empty or null value ");
            return null;
        }

        // create the Authorization header value
        String authzHdr = "Basic " + basicAuthnB64;
        BasicService httpSvc = new BasicService(authzHdr);

        // Fetch the JWT
        String href = hostinfo + authnSvcBaseURL;
        String[] headers = new String[SIZE];

        try {
            response = httpSvc.processHTTPRequest(href, RequestType.POST, null, headers);
        } catch (Exception e) {
            response = null;
        }

        if ((null != response) && (!response.isEmpty())) {
            SessionInfo info = SessionInfo.fromJson(response);
            sessionSpec = info.getSessionKey();
            // Log request header
            LogFile("Request:\n--");
            LogFile("POST " + href + " HTTP/1.1");
            LogFile(headers[REQ_HDR]);

            LogFile("Response from Auth Server:\n--");
            LogFile(headers[RES_HDR]);
            LogFile(headers[RES_BODY]);
            LogFile("");

            return sessionSpec;
        }

        return null;

    } // end getJWT

    /*
     * This method shows the most common HTTP requests that can be made to the
     * REST API. This method reads an object, updates it, creates a new object,
     * and deletes the object
     */
    public void doManage(String hostname, String baseURL) {
        print("\n===========================================================================");
        print("\nBegin Manage");
        // get the Authorization header value ready for sending to the REST API
        // It is sent as a bearer token
        String authzHdr = BEARER + sessionSpec;
        String hostinfo = hostname;
        Client client = new Client(authzHdr, baseURL, logging);

        String timestamp = new Date().toString().replaceAll(" ", "_");

        try {            
            // ---------------------------------------------------------
            LogFile("\n================Step 1=====================================================");
            LogFile("              POST: create simple domain object.");
            LogFile("===========================================================================");
            ObjectData domainData = new ObjectData();
            String name = "d_" + timestamp;
            domainData.put("Name", name);
            domainData.put("Desc", "Simple Domain created for demo");
            // POST --> create new domain with name starting with d_
            
            SmObject smdomain = client.postObjectData(null, "SmDomains", domainData, hostinfo);
            if (smdomain != null) {
                print("\n" + smdomain.getSelf() + "  posted.");
            } else {
                print("\nError: invalid response from server. Aborting.");
                return;
            }

            // ---------------------------------------------------------
            LogFile("\n================Step 2=====================================================");
            LogFile("    GET: fetch domain object and its JSON representation");
            LogFile("===========================================================================");
            String href = hostinfo + baseURL + "/SmDomains/" + name;
            // GET --> fetch existing domain by name
            smdomain = client.getObject(href);
            if (smdomain != null) {
                print("\n" + smdomain.getSelf() + "  retrieved.");
            } else {
                print("\nError: invalid response from server. Aborting.");
                return;
            }

            // -------------------------------------------------------------
            LogFile("\n================Step 3=====================================================");
            LogFile("          PUT: modify the domain object");
            LogFile("===========================================================================");
            smdomain.put("Desc", timestamp);
            smdomain.put("Mode", 0);
            // PUT --> modify desc property
            client.putObject(smdomain);
            print("\n" + smdomain.getSelf() + "  updated.");
            
            // -----------------------------------------------------------
            LogFile("\n================Step 4=====================================================");
            LogFile("        GET: retrieve list of child objects");
            LogFile("===========================================================================");
            
            // first create policy and set domain object as parent
            LogFile("/First create policy and set domain object as parent/\n");
            String newTime = new Date().toString().replaceAll(" ", "_");
            String newName = "r_" + newTime;
            ObjectData data = new ObjectData();
            data.put("Name", newName);
            data.put("IsEnabled", true);
            data.put("AllowAccess", true);

            //pass in domain as parent
            SmObject result = client.postObjectData(smdomain, "SmPolicies", data, hostinfo);
            if (result != null) {
                print("\n" + result.getSelf() + " policy posted.");
            } else {
                print("\nError: invalid response from server. Aborting.");
                return;
            }
            
            LogFile("/Retrieve link map of child objects/\n");
            LinkMap children = client.getLinkMap(smdomain.getSelf() + "/children");
            if (children != null) {
                print("\n" + smdomain.getSelf() + "/children map retrieved.");
            } else {
                print("\nError: invalid response from server. Aborting.");
                return;
            }

            // -------------------------------------------------------------
            LogFile("\n================Step 5=====================================================");
            LogFile(" GET: do a wildcard search for SmDomain objects with name starting with 'd_'");
            LogFile("===========================================================================");
            String pattern = hostinfo + baseURL + "/SmDomains/d_*";
            LinkList list = client.getLinkList(pattern);
            if (list == null) {
                print("\nEmpty list retrieved.");
            } else {
                print("\n" + pattern + " list retrieved.");

                LogFile("\n================Step 6=====================================================");
                LogFile("       DELETE: delete all objects with names starting with 'd_'");
                LogFile("===========================================================================");
                for (ObjectLink link : list.getLinks()) {
                    if (link == null)
                        continue;
                    String url = link.href;
                    // DELETE all domains with name starting with "d_". Children are also deleted
                    client.deleteObject(url);
                    print("\n" + url + "  deleted.");
                }
            }

        } catch (Exception e) {
            print("Error: ");
            e.printStackTrace();
            return;
        }
    }

    /*
     * Export policy objects to a specified file
     */
    public void doExport(String hostname, String baseURL, String passPhrase, String filepath) {
        print("\n===========================================================================");
        print("\nExporting...");

        // get the Authorization header value ready for sending to the REST API
        // It is sent as a bearer token
        String authzHdr = BEARER + sessionSpec;
        String hostinfo = hostname;
        String deploymentServiceURL = baseURL + "/deployment/export";
        Client client = new Client(authzHdr, deploymentServiceURL, logging);

        String passPhraseToken = "(PassPhrase)";
        String outFileToken = "(output.xml)";
        String payLoadString = "{\"mainObjectsMethod\":\"ADD\",\"closureObjectsMethod\":\"ADD\",\"passPhrase\":\"(PassPhrase)\",\"suggestedFileName\":\"(output.xml)\",\"objects\":[{\"path\":\"SmDomains/FedBackChannelBasicDomain\"},{\"path\":\"smagenttypes/Web Agent\"}]}";

        try {
            // ---------------------------------------------------------
            // POST --> post the Export Request
            LogFile("\n===========================================================================");
            LogFile("    POST: Send the Export Request");
            LogFile("===========================================================================\n");

            // Modify the input JSON
            payLoadString = payLoadString.replace(passPhraseToken, passPhrase);
            payLoadString = payLoadString.replace(outFileToken, filepath);

            String response = client.postData(payLoadString, hostinfo);
            if(!response.isEmpty()) {
                // write the response to a file
                WriteStringToFile(filepath, response);
                print("\nExported xml written to file.\n");
            } else {
                print("\nError: invalid response from server. Aborting.");
                return;
            }

        } catch (Exception e) {
            print("Error: ");
            e.printStackTrace();
            return;
        }
    }

    /*
     * Import an XML file using the specified url
     */
    public void doImport(String hostname, String baseURL, String passphrase, String filepath) {
        print("\n===========================================================================");
        print("\nImporting...");

        // get the Authorization header value ready for sending to the REST API
        // It is sent as a bearer token
        String authzHdr = BEARER + sessionSpec;
        String hostinfo = hostname;
        String deploymentServiceURL = baseURL + "/deployment/import";
        Client client = new Client(authzHdr, deploymentServiceURL, logging);
        StringBuilder requestBuilder = new StringBuilder();

        try {
            // ---------------------------------------------------------
            // POST --> post the Import Request
            LogFile("\n==========================================================================");
            LogFile("    POST: Send the Import Request");
            LogFile("===========================================================================\n");

            File file = new File(filepath);
            requestBuilder.append(passphrase);
            requestBuilder.append("\n");
            readFile(file, requestBuilder);

            String response = client.postData(requestBuilder.toString(), hostinfo);
            
        } catch (Exception e) {
            print("Error: ");
            e.printStackTrace();
            return;
        }
    }

    // Print usage instructions
    public void usageMessage() {
        StringBuilder helpmsg = new StringBuilder();
        helpmsg.append("\nUsage:\n run manage -ap <Authentication Service endpoint> -ep <REST endpoint> [-i for interactive login]");
        helpmsg.append("\n run export -ap <Authentication Service endpoint> -ep <REST endpoint> -pphrase <passphrase> -file <output filename> [-i for interactive login]");
        helpmsg.append("\n run import -ap <Authentication Service endpoint> -ep <REST endpoint> -pphrase <passphrase> -file <input filename> [-i for interactive login]");
        helpmsg.append("\n run help [run h]........................print usage\n");
        helpmsg.append("\n eg. run manage -ap https://localhost:8443/ca/api/sso/services/login/v1/token -ep https://localhost:8443/ca/api/sso/services/policy/v1\n");
        String help = helpmsg.toString();
        print(help);
    }


    /*
    * Parse user-entered command and arguments and store values
    */
    public boolean parseInput(String[] tokens, ParseArgs args) throws IOException {
        String authSvcEndpoint = tokens[IOF_AUTH];
        String restSvcEndpoint = tokens[IOF_REST];
        int IOF_PREFIX = authSvcEndpoint.indexOf(":");

        if (!(tokens[IOF_AP].equals(AUTHENDPOINT_SWITCH) && tokens[IOF_EP].equals(RESTENDPOINT_SWITCH))
                || !authSvcEndpoint.contains("//") || !authSvcEndpoint.substring(IOF_PREFIX + 1).contains(":") 
                || !restSvcEndpoint.contains("//") || !restSvcEndpoint.substring(IOF_PREFIX + 1).contains(":")) {
            usageMessage();
            print(INVALID_INPUT);
            return false;
        }
        args.command = tokens[IOF_COMMAND];
                
        if(authSvcEndpoint.startsWith(HTTP_PREFIX) || restSvcEndpoint.startsWith(HTTP_PREFIX)) {
            print(HTTPS_REQUIRED);
            return false;
        }
        
        // get the Authentication Service endpoint info
        URL aURL = new URL(authSvcEndpoint);
        String protocol = aURL.getProtocol();
        args.authnSvcHost = aURL.getAuthority();
        args.authnSvcHost = protocol + "://" + args.authnSvcHost;
        args.authnSvcBaseUrl = aURL.getPath();
        if(args.authnSvcBaseUrl.contains(" ")) {
            args.authnSvcBaseUrl = args.authnSvcBaseUrl.substring(0, args.authnSvcBaseUrl.indexOf(" "));
        }

        // get the REST Service endpoint info
        URL epURL = new URL(restSvcEndpoint);
        protocol = epURL.getProtocol();
        args.hostname = epURL.getAuthority();
        args.hostname = protocol + "://" + args.hostname;
        args.baseUrl = epURL.getPath();
        if(args.baseUrl.contains(" ")) {
            args.baseUrl = args.baseUrl.substring(0, args.baseUrl.indexOf(" "));
        }

        // verify export and import commands have all required arguments
        if (args.command.equals(EXPORT) || args.command.equals(IMPORT)) {
            if (tokens.length < IOF_FILENAME + 1) {
                usageMessage();
                print(INVALID_INPUT);
                return false;
            } else if (!(tokens[IOF_PPHRASE].equals(PASSPHRASE_SWITCH) && tokens[IOF_FILE].equals(FILE_SWITCH))) {
                usageMessage();
                print(INVALID_INPUT);
                return false;
            }

            // parse passphrase for import/export
            String passPhraseSub = tokens[IOF_PASSPHRASE];
            if(passPhraseSub.contains(" ")) {
                args.passPhrase = passPhraseSub.substring(0, passPhraseSub.indexOf(" "));
            } else {
                args.passPhrase = passPhraseSub;
            }
            
            // parse filename for import/export
            String fileNameSub = tokens[IOF_FILENAME];
            if(fileNameSub.contains(" ")) {
                args.fileName = fileNameSub.substring(0, fileNameSub.indexOf(" "));
            } else {
                args.fileName = fileNameSub;
            }
        }

        args.trim();
        return true;
    }

    /*
     * Main entry point to the sample
     */
    public static void main(String[] args) {
        BasicMigration s = new BasicMigration();
        s.run(args);
    } 

    /*
    * Parse command and trigger authentication against Administrative Token API. If
    * authenticated, execute the command
    */
    public void run(String[] args) {
        ParseArgs cmdLine = new ParseArgs();
        try {
            if (args.length < 1) {
                usageMessage();
                print(INVALID_INPUT);
                return;
            }
            
            if (args[IOF_COMMAND].equalsIgnoreCase("help") || args[IOF_COMMAND].equalsIgnoreCase("h")) {
                usageMessage();
                return;
            } else if (args.length > 4) {
                // Parse user command line input
                if (!parseInput(args, cmdLine)) {
                    print("Exiting.");
                    return;
                }
            } else {
                usageMessage();
                print(INVALID_INPUT);
                return;
            }
            // Whether to use properties file to capture admin credentials or interactive command line prompts.
            boolean usePropertiesFile = true;
            // '-i' triggers interactive login mode
            if (args[args.length - 1].equals("-i"))
                usePropertiesFile = false;

            print("\n===========================================================================");
            print("Begin demonstration: HTTPS requests against a REST Server.");
            LogFile("\nBegin demonstration: HTTPS requests against a REST Server.");
            print("\nSee 'output.txt' to examine HTTPS requests and responses for each REST request.\n\n");
            if (this.performAuthentication(cmdLine.authnSvcHost, cmdLine.authnSvcBaseUrl, usePropertiesFile)) {
                if (cmdLine.command.equals(MANAGE)) {
                    // complete a cycle of RESTful CRUD requests
                    this.doManage(cmdLine.hostname, cmdLine.baseUrl);
                } else if (cmdLine.command.equals(EXPORT)) {
                    // perform an export via REST
                    this.doExport(cmdLine.hostname, cmdLine.baseUrl, cmdLine.passPhrase, cmdLine.fileName);
                } else if (cmdLine.command.equals(IMPORT)) {
                    // perform an import operation via REST
                    this.doImport(cmdLine.hostname, cmdLine.baseUrl, cmdLine.passPhrase, cmdLine.fileName);
                }
            }
        } catch (Exception e1) {
            print("Error reading and parsing command:");
            e1.printStackTrace();
        }

        try {
            if (logging != null)
                logging.close();
            print("Log file closed.");
        } catch (IOException e) {
            print("Error closing the log file: " + e);
            print("Failed to close the file writer");
        }

        print("\nFinished.");
    }

    /*
     * Log errors or status messages
     */
    private static void print(String msg) {
        System.out.println(msg);
    }

    /*
     * Log output to a default text file
     */
    private static void LogFile(String str) {
        try {
            String s = str.replaceAll("\n", "\r\n");
            logging.write(s);
            logging.newLine();
            logging.flush();
        } catch (IOException ex) {
            print("Error printing to log file: " + ex);
            return;
        }
    }

    /*
     * Write a string to a new file and close the file
     * eg. write http export response to an xml file
     */
    private static void WriteStringToFile(String filePath, String str) {

        try {
            PrintWriter writer = new PrintWriter(filePath, "UTF-8");
            writer.println(str);
            writer.close();
        } catch (IOException ex) {
            print("Error creating file: " + ex);
            return;
        }
    }
}

// EOF BasicMigration.java
