/*
 * Copyright (c) 2016 CA.  All rights reserved.
 *
 * This software and all information contained therein is confidential
 * and proprietary and shall not be duplicated, used, disclosed or
 * disseminated in any way except as authorized by the applicable
 * license agreement, without the express written permission of CA.
 * All authorized reproductions must be marked with this language.
 *
 * EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE
 * EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE
 * WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT WILL CA BE LIABLE TO THE END USER OR ANY
 * THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM THE
 * USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST PROFITS,
 * BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
 * EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
 *
 */

package sample.restservice;

import java.net.HttpURLConnection;
import javax.net.ssl.HttpsURLConnection;
import java.net.URL;
import java.io.Writer;
import java.io.StringWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;
import java.util.Set;
import java.util.List;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import java.security.cert.X509Certificate;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

/**
 * This class provides an HTTP Service for interpreting calls to the REST API from the 'main' sample,
 * sending HTTP requests to the REST server, and writing the JSON response to a StringWriter.
 * It is also used to send the administrator's credentials to the Authentication Service to fetch
 * the session specification.
 *
 */
public class BasicService {
    //response codes
    private static final int    BAD_REQUEST = 400;

    //constants for logging purposes
    private static final int   REQ_HDR  = 0;
    private static final int   REQ_BODY = 1;
    private static final int   RES_HDR  = 2;
    private static final int   RES_BODY = 3;

    private HttpsURLConnection   con;    // The connection object
    private String              authorizationHdrValue;  // Credentials sent through the 
                                                        // Authorization header

    // initialize BasicService with correctly formatted base URL for the REST endpoint
    public BasicService(String authzHdrValue) {
         authorizationHdrValue = authzHdrValue;
    }
      
    /* 
    * Use an HttpsUrlConnection to build and execute HTTP requests. 
    * Request and Response headers and messages are stored in an array 
    */
    public String processHTTPRequest(String path, RequestType type, String body, String[] data) {
        URL url;
        Writer out = new StringWriter();
        
        boolean getHdz = true;
        // skip storing HTTP headers and messages if 'data' array is null
        if (data == null)
           getHdz = false;
        
        //open a connection to the requested path
        try {
          //1)create URL object from requested path, open a Connection to it
          url = new URL(path);
          con = (HttpsURLConnection) url.openConnection(); 
          
          /* To use "https" from a Java client, we skip the certificate validation
          * and hostname checks. This effectively "trusts" any SSL certificate.
          * To use this sample securely with a real self-signed cert,
          * remove this chunk of code and follow instructions in the README.
          */
          OpenTrustManager.apply((HttpsURLConnection)con);
          ((HttpsURLConnection)con).setHostnameVerifier(
                  new HostnameVerifier(){
                      public boolean verify(String arg0, SSLSession arg1) {
                          return true;
                      }
                  }
              );

        } catch (MalformedURLException e) {
          System.out.println("malformed URL: " + e);
          return "";
        } catch (IOException e) {
          System.out.println("Failed to open connection: " + e);
          return "";
        } catch (KeyManagementException e) {
            System.out.println("Certificate Error while opening a connection: " + e);
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            System.out.println("Algorithm Error while opening a connection: " + e);
            e.printStackTrace();
        }

        int responseCode = BAD_REQUEST;
        try {
          //2) determine which HTTP request to execute
          con.setRequestMethod(type.toString());
          //3) specify which content type to accept
          con.setRequestProperty("Accept", "application/json");
          // set the Authorization header
          con.setRequestProperty("Authorization", authorizationHdrValue);

          //4)for each HTTP request type, execute request and get response code
          if (type == RequestType.GET) {
            //print HTTP request header
            if (getHdz) {
              data[REQ_HDR] = getRequestHeader(con);
              data[REQ_BODY] = "";
            }
            responseCode = con.getResponseCode();

          } else if (type == RequestType.PUT) {
            //4a) if HTTP request requires content in the body,
            //specify which content type and write to connection output stream
            con.setRequestProperty("Content-Type", "application/json");
            if (getHdz) {
               //print HTTP request header and body
               data[REQ_HDR] = getRequestHeader(con);
               data[REQ_BODY] = body;
            }
            con.setDoOutput(true);
            OutputStreamWriter outputStream = new OutputStreamWriter(con.getOutputStream());
            outputStream.write(body);
            outputStream.flush();
            outputStream.close();

            responseCode = con.getResponseCode();

          } else if (type == RequestType.POST) {
            if (null != body) {
              con.setRequestProperty("Content-Type", "application/json");
            }

            //Note: getRequestHeader must occur before HttpsURLConnection.setDoOutput to avoid errors
            if (getHdz) {
              //populate array to print HTTP request header and body
              data[REQ_HDR] = getRequestHeader(con);
              data[REQ_BODY] = body;
            }

            if (null != body) {
                //write to connection output stream
                con.setDoOutput(true);
                OutputStreamWriter outputStream = new OutputStreamWriter(con.getOutputStream());
                outputStream.write(body);
                outputStream.flush();
                outputStream.close();
            }

            responseCode = con.getResponseCode();

          } else if (type == RequestType.DELETE) {
              if (getHdz) {
                //print HTTP request header and body
                data[REQ_HDR] = getRequestHeader(con);
                data[REQ_BODY] = "";
              }

              responseCode = con.getResponseCode();
          }

          if (getHdz) {
             //print HTTP response header
             data[RES_HDR] = getResponseHeader(con);
          }
        } catch (IOException i) {
          System.out.println("Error while sending request:\n" + i);
          return "";

        } catch (IllegalStateException ie) {
             System.out.println("Error setting connection properties: " + ie);
             return "";
        }

        BufferedReader reader;
        String line;

        //5)capture HTTP response body from connection and write to StringWriter
        try {
          // due to Bug in JDK #4513568 input stream is aborted on 400 error
          if (responseCode < 400) {
              reader = new BufferedReader(new InputStreamReader(con.getInputStream()));
          } else {
              reader = new BufferedReader(new InputStreamReader(con.getErrorStream()));
              String message = con.getResponseMessage();
              System.out.println("\nFailed. Error returned from server: " + responseCode + " " + message + "\n");
          }
          line = reader.readLine();

          //append each line of HTTP response to Writer to be printed in the Client
          while (line != null) {
            out.append(line);
            out.append('\n');
            line = reader.readLine();
          }
        } catch (IOException i) {
            System.out.println("Error while receiving HTTP response:\n" + i);
            i.printStackTrace();
            return "";
        }

        //6) disconnect when finished
        con.disconnect();
        //populate array with response body
        if (getHdz)
            data[RES_BODY] = out.toString();
            
        return out.toString();
      }

      //print headers in http response sent through the connection
      public String getResponseHeader(HttpsURLConnection con) {
        Map<String, List<String>> header = null;
        StringBuilder sb = new StringBuilder();
        try {
          //get header fields of response
          header = con.getHeaderFields();
          //iterate through keys to print each header field
          if (header != null) {
              Set<String> keys = header.keySet();
              for (String k : keys) {
                  if (k == null) {
                    sb.append(header.get(k).toString());
                    sb.append("\n");
                 } else {
                    sb.append(k + "  " + header.get(k).toString());
                    sb.append("\n");
                 }
              }
           }
           sb.append("\n");

        } catch (Exception ex) {
           System.out.println("Failed to retrieve response header: " + ex);
           return "";
        }

        return sb.toString();
     }

     //print headers in http request sent through the connection
     public String getRequestHeader(HttpsURLConnection con) {
       Map<String, List<String>> header = null;
       StringBuilder sb = new StringBuilder();
       try {
         //get header fields of request
         header = con.getRequestProperties();
         if (header != null) {
           Set<String> keys = header.keySet();
           for (String k : keys) {
             if (k == null) {
               sb.append(header.get(k).toString());
               sb.append("\n");
             } else {
               sb.append(k + "  " + header.get(k).toString());
               sb.append("\n");
             }
          }
       }
       sb.append("\n");

       } catch (Exception ex) {
         System.out.println("Failed to retrieve request header: " + ex);
         return "";
      }

      return sb.toString();

   }

  // The following class was derived from 
  // http://www.devsumo.com/technotes/2014/01/java-trusting-https-server-certificates/
  // This allows us to use a self-signed certificate
  public static class OpenTrustManager implements X509TrustManager {
      
      @Override
      public void checkClientTrusted(X509Certificate[] arg0, String arg1) {
      }
   
      @Override
      public void checkServerTrusted(X509Certificate[] arg0, String arg1) {
      }
   
      @Override
      public X509Certificate[] getAcceptedIssuers() {
          return null;
      }
   
      public static void apply(HttpsURLConnection conn)
              throws NoSuchAlgorithmException, KeyManagementException {
          SSLContext sc = SSLContext.getInstance("TLS");
          sc.init(null, new TrustManager[] { new OpenTrustManager() },
                  new java.security.SecureRandom());
          conn.setSSLSocketFactory(sc.getSocketFactory());
      }
  }  
}
